"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = arrayToTree;

function _react() {
  const data = _interopRequireDefault(require("react"));

  _react = function _react() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const defaultConfig = {
  id: 'id',
  parentId: 'parentId',
  dataField: 'data'
};
/**
 * Unflattens an array to a tree with runtime O(n)
 */

function arrayToTree(items, config = {}) {
  const conf = _objectSpread({}, defaultConfig, {}, config); // the resulting unflattened tree


  const rootItems = []; // stores all already processed items with ther ids as key so we can easily look them up

  const lookup = {}; // idea of this loop:
  // whenever an item has a parent, but the parent is not yet in the lookup object, we store a preliminary parent
  // in the lookup object and fill it with the data of the parent later
  // if an item has no parentId, add it as a root element to rootItems

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = items[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      const item = _step.value;
      const itemId = item[conf.id];
      const parentId = item[conf.parentId]; // look whether item already exists in the lookup table

      if (!Object.prototype.hasOwnProperty.call(lookup, itemId)) {
        // item is not yet there, so add a preliminary item (its data will be added later)
        lookup[itemId] = {
          children: []
        };
      } // add the current item's data to the item in the lookup table


      if (conf.dataField) {
        lookup[itemId][conf.dataField] = item;
      } else {
        lookup[itemId] = _objectSpread({}, item, {
          children: lookup[itemId].children
        });
      }

      const TreeItem = lookup[itemId];

      if (parentId === null) {
        // is a root item
        rootItems.push(TreeItem);
      } else {
        // has a parent
        // look whether the parent already exists in the lookup table
        if (!Object.prototype.hasOwnProperty.call(lookup, parentId)) {
          // parent is not yet there, so add a preliminary parent (its data will be added later)
          lookup[parentId] = {
            children: []
          };
        } // add the current item to the parent


        lookup[parentId].children.push(TreeItem);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return rootItems;
}